@@ -21,26 +21,61 @@
 
 
 //Read header
-bool HTTPHandler::ReadHeader( string *headerT )
+bool HTTPHandler::ReadHeader( string &headerT )
 {
+    headerT = "";
 
-    string::size_type position;
-    bool WrongHeader = false;
-    int poscount = 0;
-    ssize_t read;
     string tempheader;
+    ssize_t read;
+    int received = 0;
+    string::size_type position;
 
-    *headerT = "";
-
-    if ( (read = SocketHandler::Recv( &tempheader, false, -1 )) <= 0 )
+    //Read initial header, ignore whitespace from beginning
+    for(;;)
     {
-        return false;
+        read = SocketHandler::Recv( tempheader, false, -1 );
+
+        if ( read < 1 )
+        {
+            return false;
+        }
+
+        received += read;
+
+        if ( (position = tempheader.find_first_not_of("\r\n\t ")) != string::npos )
+        {
+            if ( position > 0 )
+            {
+                tempheader.erase( 0, position );
+
+                read -= position;
+
+                string ws;
+                SocketHandler::RecvLength( ws, position );
+            }
+
+            //Jump to header processing
+            break;
+        }
+
+        if ( received > 20480 )
+        {
+            LogFile::ErrorMessage("Too large header received (>20kB)\n");
+            return false;
+        }
+
+        SocketHandler::RecvLength( tempheader, read );
+
+        tempheader = "";
     }
 
-    while ( (position = tempheader.find ("\r\n\r\n")) == string::npos )
+    bool WrongHeader = false;
+    int poscount = 0;
+
+    while ( (position = tempheader.find("\r\n\r\n")) == string::npos )
     {
         //Maybe we should also look for \n\n (19.3 RFC 1945 - Tolerant Applications)
-        if ( (position = tempheader.find ("\n\n")) != string::npos )
+        if ( (position = tempheader.find("\n\n")) != string::npos )
         {
             WrongHeader = true;
             break;
@@ -54,19 +89,25 @@ bool HTTPHandler::ReadHeader( string *headerT )
         }
 
         poscount += read;
+        received += read;
 
         //Too big header
-        if ( poscount > 20480 )
+        if ( received > 20480 )
         {
-            LogFile::ErrorMessage("Server sent too big header (>20kB)\n");
+            LogFile::ErrorMessage("Too large header received (>20kB)\n");
             return false;
         }
 
-        if ( (read = SocketHandler::Recv( &tempheader, false, -1 )) <= 0 )
+        if ( (read = SocketHandler::Recv( tempheader, false, -1 )) < 1 )
         {
+            //Did not receive last empty line?
+            if ( read == 0 && ( headerT.find_last_of( "\r\n" ) == headerT.size() - 1 ) )
+            {
+                return true;
+            }
+
             return false;
         }
-
     }
 
     //Read last part of header
@@ -78,7 +119,7 @@ bool HTTPHandler::ReadHeader( string *headerT )
     if ( WrongHeader == false )
     {
         //Read last \r\n
-        if ( SocketHandler::RecvLength(&tempheader, 2 ) == false )
+        if ( SocketHandler::RecvLength( tempheader, 2 ) == false )
         {
             return false;
         }
@@ -89,7 +130,7 @@ bool HTTPHandler::ReadHeader( string *headerT )
 
 
 //Split header to tokens
-int HTTPHandler::AnalyseHeader( string *linesT )
+int HTTPHandler::AnalyseHeader( string &linesT )
 {
 
     //Delete header tokens
@@ -98,17 +139,17 @@ int HTTPHandler::AnalyseHeader( string *linesT )
     string::size_type lastposition = 0;
 
     //Do "Tolerant Applications" - RFC 1945 - Hypertext Transfer Protocol -- HTTP/1.0
-    while ( (lastposition = linesT->find( "\r", lastposition )) != string::npos )
+    while ( (lastposition = linesT.find( "\r", lastposition )) != string::npos )
     {
-        linesT->replace( lastposition, 1, "" );
+        linesT.replace( lastposition, 1, "" );
     }
 
     lastposition = 0;
 
-    string::size_type length = linesT->length();
+    string::size_type length = linesT.length();
     string::size_type position, positiontmp;
     
-    if ( (position = linesT->find( "\n", 0 )) == string::npos )
+    if ( (position = linesT.find( "\n", 0 )) == string::npos )
     {
         return -201;
     }
@@ -122,7 +163,7 @@ int HTTPHandler::AnalyseHeader( string *linesT )
     //Loop through headers
     while ( position != string::npos && lastposition != length )
     {
-        tempToken = linesT->substr( lastposition, position - lastposition );
+        tempToken = linesT.substr( lastposition, position - lastposition );
 
         if ( (lastposition = tempToken.find_last_not_of("\t ")) != string::npos )
         {
@@ -131,7 +172,7 @@ int HTTPHandler::AnalyseHeader( string *linesT )
             if ( First == true )
             {
                 //Analyse request header
-                if ( (ret = AnalyseFirstHeaderLine( &tempToken )) < 0 )
+                if ( (ret = AnalyseFirstHeaderLine( tempToken )) < 0 )
                 {
                     return ret;
                 }
@@ -142,13 +183,14 @@ int HTTPHandler::AnalyseHeader( string *linesT )
                 if ( (positiontmp = tempToken.find(":")) != string::npos )
                 {
                     headerbase = tempToken.substr(0, positiontmp + 1);
+                    headerbase += " ";
 
                     //Make sure we have "Header:<SPACE>value"
                     if ( (positiontmp = tempToken.find_first_not_of(" ", positiontmp + 1)) != string::npos )
                     {
-                        tempToken = headerbase + " " + tempToken.substr(positiontmp);
+                        tempToken = headerbase + tempToken.substr(positiontmp);
 
-                        if ( (ret = AnalyseHeaderLine( &tempToken )) < 0 )
+                        if ( (ret = AnalyseHeaderLine( tempToken )) < 0 )
                         {
                             return ret;
                         }
@@ -162,7 +204,7 @@ int HTTPHandler::AnalyseHeader( string *linesT )
         }
 
         lastposition = position + 1;
-        position = linesT->find( "\n", lastposition );
+        position = linesT.find( "\n", lastposition );
     }
 
     return 0;
@@ -170,10 +212,9 @@ int HTTPHandler::AnalyseHeader( string *linesT )
 
 
 //Read part of Body
-ssize_t HTTPHandler::ReadBodyPart( string* bodyT )
+ssize_t HTTPHandler::ReadBodyPart( string &bodyT )
 {
-
-    *bodyT = "";
+    bodyT = "";
     ssize_t count;
 
     if ( (count = SocketHandler::Recv( bodyT, true, -1 )) < 0)
@@ -196,10 +237,10 @@ bool HTTPHandler::SendHeader( string header, bool ConnectionClose )
     }
     else
     {
-        header += "Connection: Keep-Alive\r\n\r\n";
+        header += "Connection: keep-alive\r\n\r\n";
     }
 
-    if ( SocketHandler::Send( &header ) == false )
+    if ( SocketHandler::Send( header ) == false )
     {
         return false;
     }
